using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace EvilAttributes.Tests
{
    [TestClass]
    public class AttributeDepthTests
    {
        [TestMethod()]
        public void ShouldFailBecauseOfComparisonToEmptyIEnumerableIList()
        {
            IListEntity entity = new IListEntity();
            entity.Property1 = 10;
            entity.IListSubEntity1 = new List<SubEntity1>();

            Assert.IsFalse(entity.IsValid(), "Entity should not be valid if the CompareTo object is IEnumerable");
            Assert.AreEqual(entity.ValidationErrors.Count, 1, "Entity should have 1 error in this case");
        }

        [TestMethod()]
        public void ShouldFailBecauseOfComparisonToPopulatedIEnumerableIList()
        {
            IListEntity entity = new IListEntity();
            entity.Property1 = 10;
            entity.IListSubEntity1 = new List<SubEntity1>();

            SubEntity1 subEntity = new SubEntity1();
            subEntity.SubProperty1 = 100;
            subEntity.SubProperty2 = 200;
            entity.IListSubEntity1.Add(subEntity);

            Assert.IsFalse(entity.IsValid(), "Entity should not be valid if the CompareTo object is IEnumerable");
            Assert.AreEqual(entity.ValidationErrors.Count, 1, "Entity should have 1 error in this case");
        }

        [TestMethod()]
        public void ShouldSuccessfullyCompareToAnIEnumerableCountProperty()
        {
            IListEntity2 entity = new IListEntity2();
            entity.Property1 = 1;
            entity.IListSubEntity1 = new List<SubEntity1>();

            SubEntity1 subEntity = new SubEntity1();
            subEntity.SubProperty1 = 100;
            subEntity.SubProperty2 = 200;
            entity.IListSubEntity1.Add(subEntity);

            SubEntity1 subEntity2 = new SubEntity1();
            subEntity2.SubProperty1 = 55;
            subEntity2.SubProperty2 = 88;
            entity.IListSubEntity1.Add(subEntity2);

            Assert.IsTrue(entity.IsValid(), "Entity is not less than the count of SubEntity objects");
            Assert.AreEqual(entity.ValidationErrors.Count, 0, "Entity should not have any errors");
        }

        [TestMethod()]
        public void ShouldFindPropertyOneAndTwoLevelsDeepAndIsValid()
        {
            Entity entity = new Entity();
            entity.Property1 = 8;
            entity.Property2 = 55;
            
            SubEntity1 subEntity1 = new SubEntity1();
            subEntity1.SubProperty1 = 6;
            subEntity1.SubProperty2 = 10;

            SubEntity2 subEntity2 = new SubEntity2();
            subEntity2.SubSubProperty1 = 100;

            subEntity1.SubProperty3 = subEntity2;

            entity.SubEntityProperty = subEntity1;

            Assert.IsTrue(entity.IsValid(), "Entity is not valid");
            Assert.AreEqual(0, entity.ValidationErrors.Count, "Entity Validation Errors are greater than 0");
        }

        [TestMethod()]
        public void ShouldFindPropertyOneAndTwoLevelsDeepAndIsInValidOneLevelDeep()
        {
            Entity entity = new Entity();
            entity.Property1 = 8;
            entity.Property2 = 55;

            SubEntity1 subEntity1 = new SubEntity1();
            subEntity1.SubProperty1 = 6;
            subEntity1.SubProperty2 = 6;

            SubEntity2 subEntity2 = new SubEntity2();
            subEntity2.SubSubProperty1 = 100;

            subEntity1.SubProperty3 = subEntity2;

            entity.SubEntityProperty = subEntity1;

            Assert.IsFalse(entity.IsValid(), "Entity is valid and should not be");
            Assert.AreEqual(1, entity.ValidationErrors.Count, "Entity Validation Errors are greater than 0");
        }

        [TestMethod()]
        public void ShouldFindPropertyOneAndTwoLevelsDeepAndIsInValidTwoLevelsDeep()
        {
            Entity entity = new Entity();
            entity.Property1 = 8;
            entity.Property2 = 55;

            SubEntity1 subEntity1 = new SubEntity1();
            subEntity1.SubProperty1 = 6;
            subEntity1.SubProperty2 = 10;

            SubEntity2 subEntity2 = new SubEntity2();
            subEntity2.SubSubProperty1 = 40;

            subEntity1.SubProperty3 = subEntity2;

            entity.SubEntityProperty = subEntity1;

            Assert.IsFalse(entity.IsValid(), "Entity is valid and should not be");
            Assert.AreEqual(1, entity.ValidationErrors.Count, "Entity Validation Errors should be 1");
        }

        [TestMethod()]
        public void ShouldFailTestingListPropertyWithValuesInTheList()
        {
            ParentListEntity entity = new ParentListEntity();
            entity.Property1 = 11;

            SubEntity2 subEntity = new SubEntity2();
            subEntity.SubSubProperty1 = 10000;
            entity.SubEntities.Add(subEntity);

            Assert.IsFalse(entity.IsValid(), "Entity is valid and should not be");
            Assert.AreEqual(1, entity.ValidationErrors.Count, "Entity Validation Errors should be 1");
        }

        [TestMethod()]
        public void ShouldFailTestingListPropertyWithNoValuesInTheList()
        {
            ParentListEntity entity = new ParentListEntity();
            entity.Property1 = 11;
            
            Assert.IsFalse(entity.IsValid(), "Entity is valid and should not be");
            Assert.AreEqual(1, entity.ValidationErrors.Count, "Entity Validation Errors should be 1");
        }

        #region Entity Classes

        private class Entity : EvilBaseEntity
        {
            private int _property1;

            [ValidateLessThan("SubEntityProperty.SubProperty2","SubEntityProperty's SubProperty2 should be greater than Property1")]
            public int Property1
            {
                get { return _property1; }
                set { _property1 = value; }
            }

            private int _property2;

            [ValidateLessThan("SubEntityProperty.SubProperty3.SubSubProperty1","SubEntity2Property's SubProperty3's SubSubProperty1 should be greater than Property2")]
	        public int Property2
	        {
		        get { return _property2;}
		        set { _property2 = value;}
	        }

            private SubEntity1 _subEntityProperty;

            public SubEntity1 SubEntityProperty
            {
                get { return _subEntityProperty; }
                set { _subEntityProperty = value; }
            }

            private SubEntity2 _subEntity2Property;

	        public SubEntity2 SubEntity2Property
	        {
		        get { return _subEntity2Property;}
		        set { _subEntity2Property = value;}
	        }
        }

        private class IListEntity : EvilBaseEntity
        {
            private int _property1;

            [ValidateLessThan("IListSubEntity1.Count","This should not be an allowable comparison")]
	        public int Property1
	        {
		        get { return _property1;}
		        set { _property1 = value;}
	        }	
	
            private IList<SubEntity1> _ilistSubentity1;

	        public IList<SubEntity1> IListSubEntity1
	        {
		        get { return _ilistSubentity1;}
		        set { _ilistSubentity1 = value;}
	        }
        }

        private class IListEntity2 : EvilBaseEntity
        {
            private int _property1;

            [ValidateLessThan("IListSubEntity1.Count", "Property1 must be less than the count of SubEntities")]
            public int Property1
            {
                get { return _property1; }
                set { _property1 = value; }
            }

            private IList<SubEntity1> _ilistSubentity1;

            public IList<SubEntity1> IListSubEntity1
            {
                get { return _ilistSubentity1; }
                set { _ilistSubentity1 = value; }
            }
        }

        private class SubEntity1
        {
            private int _subProperty1;

            public int SubProperty1
            {
                get { return _subProperty1; }
                set { _subProperty1 = value; }
            }

            private int _subProperty2;

            public int SubProperty2
            {
                get { return _subProperty2; }
                set { _subProperty2 = value; }
            }

            private SubEntity2 _subProperty3;

	        public SubEntity2 SubProperty3
	        {
		        get { return _subProperty3;}
		        set { _subProperty3 = value;}
	        }
        }

        private class SubEntity2
        {
            private int _subSubProperty1;

	        public int SubSubProperty1
	        {
		        get { return _subSubProperty1;}
		        set { _subSubProperty1 = value;}
	        }
        }

        private class ParentListEntity : EvilBaseEntity
        {
            private int _property1;

            [ValidateLessThan("SubEntities.Count","Property1 should be less than SubEntities")]
            public int Property1
            {
                get { return _property1; }
                set { _property1 = value; }
            }

            private IList<SubEntity2> _subEntities = new List<SubEntity2>();

            public IList<SubEntity2> SubEntities
            {
                get { return _subEntities; }
                set { _subEntities = value; }
            }

        }

        #endregion
    }
}
