using System;
using System.Collections.Generic;
using System.Reflection;
using System.Text;

namespace EvilAttributes
{
    /// <summary>
    /// Abstract class that all EViL attributes must inherit from.
    /// </summary>
    public abstract class EvilBaseAttribute : Attribute, IValidator
    {
        private string _message;

        /// <summary>
        /// Gets the message for the attribute.
        /// </summary>
        public string Message
        {
            get { return _message; }
        }

        /// <summary>
        /// Base constructor for all attributes that inherit from this abstract class.
        /// </summary>
        /// <param name="message">The message to be used if validation fails.</param>
        protected EvilBaseAttribute(string message)
        {
            _message = message;
        }

        /// <summary>
        /// Abstract method. Validates a property value of an entity.
        /// </summary>
        /// <param name="value">The value of the property to validate.</param>
        /// <param name="entity">The entity that contains the property value.</param>
        /// <returns>True if the property value is valid, false otherwise.</returns>
        public abstract bool ValidateValue(object value, object entity);

        /// <summary>
        /// Gets the value of property that is being compared to. This recursively searches
        /// through the object tree based on the propertyName passed in. If the name is in
        /// a Class.Class.Class.Property format it will traverse all the way to the property
        /// and use it for comparison.
        /// </summary>
        /// <param name="propertyName">String representation of the property that is being compared to.
        /// Should be in a Class.Class.Class.Property format if the property is in embedded objects.</param>
        /// <param name="entity">The class that the property is being searched for in.</param>
        /// <returns>The property value if it is found. If the property is not found, null is returned.</returns>
        protected object GetPropertyValue(string propertyName, object entity)
        {
            string[] propertyNames = propertyName.Split('.');

            if (propertyNames.Length == 0 || entity == null)
            {
                return null;
            }

            string firstPropertyName = propertyNames[0];

            IList<ISmartMemberInfo> infos = PropertyFactory.GetProperties(entity);

            // Loop thru to find the one we want
            foreach (ISmartMemberInfo smi in infos)
            {
                if (smi.Name == firstPropertyName)
                {
                    return DeterminePropertyValue(entity, smi, propertyNames);
                }
            }

            return null;
        }

        private object DeterminePropertyValue(object entity, ISmartMemberInfo smartMemberInfo, string[] propertyNames)
        {
            // This is to prevent comparisons against lists of objects, but allows comparison
            // against properties of those lists.
            if (smartMemberInfo.PropertyType.GetInterface("IEnumerable") != null && propertyNames.Length == 1)
            {
                return GetIEnumerablePropertyValue(entity, smartMemberInfo);
            }
            else if (propertyNames.Length > 1)
            {
                return GetPropertyValue(MakeTrimmedPropertyName(propertyNames), smartMemberInfo.GetValue(entity, null));
            }
            else
            {
                return smartMemberInfo.GetValue(entity, null);
            }
        }

        private object GetIEnumerablePropertyValue(object entity, ISmartMemberInfo smartMemberInfo)
        {
            // This is needed because strings are IEnumerable and without this we can't compare
            // string to string easily.
            if (typeof(string) == smartMemberInfo.PropertyType)
            {
                return smartMemberInfo.GetValue(entity, null);
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// Manages trimming the first value off of the Class.Class.Class.Property syntax that can be used.
        /// </summary>
        /// <param name="properties">A string array of the Class.Class.Class.Property format that has been split based on a '.'</param>
        /// <returns>The trimmed property list reassembled in a Class.Class.Class.Property format.</returns>
        private string MakeTrimmedPropertyName(string[] properties)
        {
            StringBuilder property = new StringBuilder();
            for (int i = 1; i < properties.Length; i++)
            {
                property.Append(properties[i]);
                if (i != properties.Length - 1)
                {
                    property.Append(".");
                }
            }

            return property.ToString();
        }
    }
}
