using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Reflection;

namespace EvilAttributes
{
    /// <summary>
    /// Used to validate an entity. This class allows an entity to be validated without having
    /// to inherit from EvilBaseEntity.
    /// </summary>
    [Serializable()]
    public class EvilValidator
    {
        private object _entity;
        private Collection<string> _validationErrors;

        /// <summary>
        /// The entity to be validated.
        /// </summary>
        public object Entity
        {
            get { return _entity; }
        }

        /// <summary>
        /// Gets the collection of validation errors for the entity.
        /// </summary>
        public Collection<string> ValidationErrors
        {
            get
            {
                if (_validationErrors == null)
                {
                    _validationErrors = new Collection<string>();
                }
                return _validationErrors;
            }
        }

        /// <summary>
        /// Initializes a new instance of the EvilValidator class.
        /// </summary>
        /// <param name="entity">The entity to be validated.</param>
        public EvilValidator(object entity)
        {
            _entity = entity;
        }

        /// <summary>
        /// Determines if the entity is valid.
        /// </summary>
        /// <returns>True if the entity is valid, false otherwise.</returns>
        /// <remarks>
        /// An entity is valid only if all attributes applied to the entity pass validation.
        /// </remarks>
        public bool IsValid()
        {
            // First clear any existing errors
            this.ValidationErrors.Clear();

            // Get all properties on the entity and validate
            IList<ISmartMemberInfo> methodInfos = PropertyFactory.GetProperties(this.Entity);
            ValidateProperties(methodInfos);

            if (this.ValidationErrors.Count > 0)
            {
                return false;
            }

            // Otherwise
            return true;
        }

        private void ValidateProperties(IList<ISmartMemberInfo> smartMemberInfos)
        {
            // Go thru all properties and validate
            foreach (ISmartMemberInfo smi in smartMemberInfos)
            {
                // Get all validation attributes on the property
                EvilBaseAttribute[] attributes = smi.GetCustomAttributes(typeof(EvilBaseAttribute), true);

                // Get the value of the property on the current entity
                object propValue = null;
                if (attributes.Length > 0)
                {
                    propValue = smi.GetValue(this.Entity, null);
                }

                // Loop thru each attribute on the property and invoke ValidateValue
                foreach (EvilBaseAttribute attrib in attributes)
                {
                    // Each attribute type implements its own ValidateValue
                    if (!attrib.ValidateValue(propValue, this.Entity))
                    {
                        this.ValidationErrors.Add(attrib.Message);
                    }
                }
            }
        }
    }
}
