using System;
using System.Reflection;

namespace EvilAttributes
{
    /// <summary>
    /// Indicates a DateTime property occurs on or before another DateTime property. This class cannot be inherited.
    /// </summary>
    [AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = true)]
    public sealed class ValidateDateOccursOnOrBeforeAttribute : EvilBaseAttribute
    {
        private string _propertyName;

        /// <summary>
        /// Gets the name of the DateTime property to compare.
        /// </summary>
        public string PropertyName
        {
            get { return _propertyName; }
        }

        /// <summary>
        /// Initializes a new instance of the ValidateDateOccursOnOrBeforeAttribute class.
        /// </summary>
        /// <param name="propertyName">The name of the DateTime property to compare.</param>
        /// <param name="message">The message to be used if validation fails.</param>
        public ValidateDateOccursOnOrBeforeAttribute(string propertyName, string message)
            : base(message)
        {
            _propertyName = propertyName;
        }

        /// <summary>
        /// Validates a property value of an entity.
        /// </summary>
        /// <param name="value">The value of the property to validate.</param>
        /// <param name="entity">The entity that contains the property value.</param>
        /// <returns>True if the property value is valid, false otherwise.</returns>
        public override bool ValidateValue(object value, object entity)
        {
            object propValue = base.GetPropertyValue(this.PropertyName, entity);

            // Do null check then do datetime comparison
            if (propValue != null)
            {
                DateTime inputDate = DateTime.Parse(value.ToString(), null);
                DateTime dateToCompare = DateTime.Parse(propValue.ToString(), null);

                if (DateTime.Compare(inputDate, dateToCompare) <= 0)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }

            // Otherwise
            return true;
        }
    }
}
