using System;
using System.Reflection;

namespace EvilAttributes
{
    /// <summary>
    /// Indicates a property is required to be null or empty if another property equals a certain value. This class cannot be inherited.
    /// </summary>
    [AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = true)]
    public sealed class ValidateRequiredNullOrEmptyIfEqualsAttribute : EvilBaseAttribute
    {
        private string _propertyName;
        private string _propertyValue;

        /// <summary>
        /// Gets the name of the property to compare.
        /// </summary>
        public string PropertyName
        {
            get { return _propertyName; }
        }

        /// <summary>
        /// Gets the value of the property to compare.
        /// </summary>
        public string PropertyValue
        {
            get { return _propertyValue; }
        }

        /// <summary>
        /// Initializes a new instance of the ValidateRequiredNullOrEmptyIfEqualsAttribute class.
        /// </summary>
        /// <param name="propertyName">The name of the property to compare.</param>
        /// <param name="propertyValue">The value of the property to compare.</param>
        /// <param name="message">The message to be used if validation fails.</param>
        public ValidateRequiredNullOrEmptyIfEqualsAttribute(string propertyName, string propertyValue, string message)
            : base(message)
        {
            _propertyName = propertyName;
            _propertyValue = propertyValue;
        }

        /// <summary>
        /// Validates a property value of an entity.
        /// </summary>
        /// <param name="value">The value of the property to validate.</param>
        /// <param name="entity">The entity that contains the property value.</param>
        /// <returns>True if the property value is valid, false otherwise.</returns>
        public override bool ValidateValue(object value, object entity)
        {
            object propValue = base.GetPropertyValue(this.PropertyName, entity);

            if (propValue != null && propValue.ToString().Length > 0)
            {
                // Check if the value matches
                if (propValue.ToString() == this.PropertyValue)
                {
                    // Now check if the property is null or empty
                    if (value == null || value.ToString().Length == 0)
                    {
                        return true;
                    }
                }
                else
                {
                    return true;
                }
            }
            else
            {
                return true;
            }

            // Otherwise
            return false;
        }
    }
}
